@@ -7,21 +7,19 @@ The `BitSlice` module discusses the design decisions for the separation between
 slice and vector types.
 !*/
 
-#![cfg(any(feature = "alloc", feature = "std"))]
+#![cfg(feature = "alloc")]
 
 use crate::{
+	access::BitAccess,
 	boxed::BitBox,
-	cursor::{
-		Cursor,
+	indices::Indexable,
+	order::{
+		BitOrder,
 		Local,
 	},
-	indices::Indexable,
 	pointer::BitPtr,
 	slice::BitSlice,
-	store::{
-		BitStore,
-		Word,
-	},
+	store::BitStore,
 };
 
 use alloc::{
@@ -32,10 +30,9 @@ use alloc::{
 use core::{
 	marker::PhantomData,
 	mem,
-	slice,
 };
 
-/** A compact [`Vec`] of bits, whose cursor and storage type can be customized.
+/** A compact [`Vec`] of bits, whose order and storage type can be customized.
 
 `BitVec` is a newtype wrapper over `Vec`, and as such is exactly three words in
 size on the stack.
@@ -139,7 +136,7 @@ a bit slice, use `&`. Example:
 ```rust
 use bitvec::prelude::*;
 fn read_bitslice(slice: &BitSlice) {
-	// use slice
+    // use slice
 }
 
 let bv = bitvec![0, 1];
@@ -147,7 +144,7 @@ read_bitslice(&bv);
 
 // … and that’s all!
 // you can also do it like this:
-let bs : &BitSlice = &bv;
+let bs: &BitSlice = &bv;
 ```
 
 In Rust, it’s more common to pass slices as arguments rather than vectors when
@@ -186,7 +183,7 @@ interpreted as anything but opaque binary data by user code.
 
 When a `BitVec` has allocated memory, then the memory to which it points is on
 the heap (as defined by the allocator Rust is configured to use by default), and
-its pointer points to [`len`] initialized bits in order of the [`Cursor`] type
+its pointer points to [`len`] initialized bits in order of the [`BitOrder`] type
 parameter, followed by `capacity - len` logically uninitialized bits.
 
 `BitVec` will never perform a “small optimization” where elements are stored in
@@ -220,7 +217,7 @@ space, then increasing the length to match, is always valid.
 
 # Type Parameters
 
-- `C: Cursor`: An implementor of the [`Cursor`] trait. This type is used to
+- `O: BitOrder`: An implementor of the [`BitOrder`] trait. This type is used to
   convert semantic indices into concrete bit positions in elements, and store or
   retrieve bit values from the storage type.
 - `T: BitStore`: An implementor of the [`BitStore`] trait: `u8`, `u16`, `u32`,
@@ -234,10 +231,10 @@ is ***extremely binary incompatible*** with them. Attempting to treat
 `BitVec<_, T>` as `Vec<T>` in any manner except through the provided APIs is
 ***catastrophically*** unsafe and unsound.
 
-[`BitSlice`]: ../struct.BitSlice.html
+[`BitSlice`]: ../slice/struct.BitSlice.html
 [`BitVec::with_capacity`]: #method.with_capacity
-[`BitStore`]: ../trait.BitStore.html
-[`Cursor`]: ../trait.Cursor.html
+[`BitStore`]: ../store/trait.BitStore.html
+[`BitOrder`]: ../order/trait.BitOrder.html
 [`Index`]: https://doc.rust-lang.org/stable/std/ops/trait.Index.html
 [`String`]: https://doc.rust-lang.org/stable/std/string/struct.String.html
 [`Vec`]: https://doc.rust-lang.org/stable/std/vec/struct.Vec.html
@@ -249,18 +246,38 @@ is ***extremely binary incompatible*** with them. Attempting to treat
 [`&[]`]: https://doc.rust-lang.org/stable/std/primitive.slice.html
 **/
 #[repr(C)]
-pub struct BitVec<C = Local, T = Word>
-where C: Cursor, T: BitStore {
-	/// Phantom `Cursor` member to satisfy the constraint checker.
-	_cursor: PhantomData<C>,
+pub struct BitVec<O = Local, T = usize>
+where O: BitOrder, T: BitStore {
+	/// Phantom `BitOrder` member to satisfy the constraint checker.
+	_order: PhantomData<O>,
 	/// Slice pointer over the owned memory.
 	pointer: BitPtr<T>,
 	/// The number of *elements* this vector has allocated.
 	capacity: usize,
 }
 
-impl<C, T> BitVec<C, T>
-where C: Cursor, T: BitStore {
+impl<O, T> BitVec<O, T>
+where O: BitOrder, T: BitStore {
+	/// Constructs a `BitVec` from a value repeated many times.
+	///
+	/// This function is equivalent to the `bitvec![O, T; bit; len]` macro call,
+	/// and is in fact the implementation of that macro syntax.
+	///
+	/// # Parameters
+	///
+	/// - `bit`: The bit value to which all `len` allocated bits will be set.
+	/// - `len`: The number of live bits in the constructed `BitVec`.
+	///
+	/// # Returns
+	///
+	/// A `BitVec` with `len` live bits, all set to `bit`.
+	pub fn repeat(bit: bool, len: usize) -> Self {
+		let mut out = Self::with_capacity(len);
+		unsafe { out.set_len(len); }
+		out.set_elements(if bit { T::TRUE } else { T::FALSE });
+		out
+	}
+
 	/// Constructs a `BitVec` from a single element.
 	///
 	/// The produced `BitVec` will span the element, and include all bits in it.
@@ -278,15 +295,14 @@ where C: Cursor, T: BitStore {
 	/// ```rust
 	/// use bitvec::prelude::*;
 	///
-	/// let bv = BitVec::<BigEndian, u8>::from_element(5);
+	/// let bv = BitVec::<Msb0, u8>::from_element(5);
 	/// assert_eq!(bv.count_ones(), 2);
 	/// ```
+	#[inline]
 	pub fn from_element(elt: T) -> Self {
-		Self::from_vec({
-			let mut v = Vec::with_capacity(1);
-			v.push(elt);
-			v
-		})
+		let mut v = Vec::with_capacity(1);
+		v.push(elt);
+		Self::from_vec(v)
 	}
 
 	/// Constructs a `BitVec` from a slice of elements.
@@ -307,14 +323,15 @@ where C: Cursor, T: BitStore {
 	/// use bitvec::prelude::*;
 	///
 	/// let src = [5, 10];
-	/// let bv = BitVec::<BigEndian, u8>::from_slice(&src[..]);
+	/// let bv = BitVec::<Msb0, u8>::from_slice(&src[..]);
 	/// assert!(bv[5]);
 	/// assert!(bv[7]);
 	/// assert!(bv[12]);
 	/// assert!(bv[14]);
 	/// ```
+	#[inline]
 	pub fn from_slice(slice: &[T]) -> Self {
-		BitSlice::<C, T>::from_slice(slice).to_owned()
+		Self::from_vec(slice.to_owned())
 	}
 
 	/// Consumes a `Vec<T>` and creates a `BitVec<C, T>` from it.
@@ -337,7 +354,7 @@ where C: Cursor, T: BitStore {
 	/// ```rust
 	/// use bitvec::prelude::*;
 	///
-	/// let bv = BitVec::<BigEndian, u8>::from_vec(vec![1, 2, 4, 8]);
+	/// let bv = BitVec::<Msb0, u8>::from_vec(vec![1, 2, 4, 8]);
 	/// assert_eq!(
 	///   "[00000001, 00000010, 00000100, 00001000]",
 	///   &format!("{}", bv),
@@ -351,12 +368,12 @@ where C: Cursor, T: BitStore {
 			len,
 			BitPtr::<T>::MAX_ELTS,
 		);
-		let bs = BitSlice::<C, T>::from_slice(&vec[..]);
+		let bs = BitSlice::<O, T>::from_slice(&vec[..]);
 		let pointer = bs.bitptr();
 		let capacity = vec.capacity();
 		mem::forget(vec);
 		Self {
-			_cursor: PhantomData,
+			_order: PhantomData,
 			pointer,
 			capacity,
 		}
@@ -364,9 +381,25 @@ where C: Cursor, T: BitStore {
 
 	/// Clones a `&BitSlice` into a `BitVec`.
 	///
+	/// This is the only method by which a `BitVec` can be created whose first
+	/// live bit is not at the `0` position. This behavior, though
+	/// unconventional for common uses of `BitVec`, allows for a more efficient
+	/// clone of any `BitSlice` region without shifting each bit in the region
+	/// down to fit the `0` starting position.
+	///
+	/// Misaligned `BitVec`s **do not** have any adverse effect on usage other
+	/// than the in-memory representation.
+	///
+	/// Whenever a `BitVec` is emptied, its head index is always set to `0`, and
+	/// will begin from the aligned position on future refills.
+	///
+	/// The [`::force_align`] method will shift the `BitVec`’s data to begin at
+	/// the `0` index, if you require this property.
+	///
 	/// # Parameters
 	///
-	/// - `slice`
+	/// - `slice`: The source `BitSlice` region. This may have any head index,
+	///   and its memory will be copied element-wise into the new buffer.
 	///
 	/// # Returns
 	///
@@ -377,15 +410,37 @@ where C: Cursor, T: BitStore {
 	/// ```rust
 	/// use bitvec::prelude::*;
 	///
-	/// let bs = [0u8, !0].bits::<BigEndian>();
+	/// let bs = [0u8, !0].bits::<Msb0>();
 	/// let bv = BitVec::from_bitslice(bs);
 	/// assert_eq!(bv.len(), 16);
 	/// assert!(bv.some());
 	/// ```
-	pub fn from_bitslice(slice: &BitSlice<C, T>) -> Self {
-		let mut out = Self::with_capacity(slice.len());
-		out.extend(slice.iter().copied());
-		out
+	///
+	/// [`::force_align`]: #method.force_align
+	pub fn from_bitslice(slice: &BitSlice<O, T>) -> Self {
+		let mut pointer = slice.bitptr();
+		let source = pointer.as_access_slice();
+
+		//  Create a blank buffer into which the source will be copied.
+		let mut v = Vec::with_capacity(source.len());
+
+		//  Copy the source into the buffer. This must be done per-element, so
+		//  that atomic systems will correctly synchronize.
+		source.iter().for_each(|elt| v.push(elt.load()));
+
+		/* Target the copied pointer to the buffer’s region, preserving its
+		length and offset information. This enables `BitVec` to efficiently
+		lift from any `&BitSlice`, without having to reälign the source per-bit.
+		*/
+		unsafe { pointer.set_pointer(v.as_ptr() as *const T); }
+
+		let capacity = v.capacity();
+		mem::forget(v);
+		Self {
+			_order: PhantomData,
+			pointer,
+			capacity,
+		}
 	}
 
 	/// Converts a frozen `BitBox` allocation into a growable `BitVec`.
@@ -409,7 +464,7 @@ where C: Cursor, T: BitStore {
 	/// assert_eq!(bv.len(), 2);
 	/// assert!(bv.some());
 	/// ```
-	pub fn from_boxed_bitslice(slice: BitBox<C, T>) -> Self {
+	pub fn from_boxed_bitslice(slice: BitBox<O, T>) -> Self {
 		let bitptr = slice.bitptr();
 		mem::forget(slice);
 		unsafe { Self::from_raw_parts(bitptr, bitptr.elements()) }
@@ -450,9 +505,10 @@ where C: Cursor, T: BitStore {
 	/// `BitVec<C, T>` which may then deallocate, reallocate, or modify the
 	/// contents of the referent slice at will. Ensure that nothing else uses
 	/// the pointer after calling this function.
+	#[inline]
 	pub unsafe fn from_raw_parts(pointer: BitPtr<T>, capacity: usize) -> Self {
 		Self {
-			_cursor: PhantomData,
+			_order: PhantomData,
 			pointer,
 			capacity,
 		}
@@ -478,7 +534,8 @@ where C: Cursor, T: BitStore {
 	/// let bv = bitvec![0, 1, 1, 0];
 	/// let bs = bv.as_bitslice();
 	/// ```
-	pub fn as_bitslice(&self) -> &BitSlice<C, T> {
+	#[inline]
+	pub fn as_bitslice(&self) -> &BitSlice<O, T> {
 		self.pointer.into_bitslice()
 	}
 
@@ -502,25 +559,28 @@ where C: Cursor, T: BitStore {
 	/// let mut bv = bitvec![0, 1, 1, 0];
 	/// let bs = bv.as_mut_bitslice();
 	/// ```
-	pub fn as_mut_bitslice(&mut self) -> &mut BitSlice<C, T> {
+	#[inline]
+	pub fn as_mut_bitslice(&mut self) -> &mut BitSlice<O, T> {
 		self.pointer.into_bitslice_mut()
 	}
 
 	/// Sets the backing storage to the provided element.
 	///
-	/// This unconditionally sets each allocated element in the backing storage
-	/// to the provided value, without altering the `BitVec` length or capacity.
-	/// It operates on the underlying `Vec`’s memory region directly, and will
-	/// ignore the `BitVec`’s cursors.
+	/// This unconditionally sets each live element in the backing buffer to the
+	/// provided value, without altering the `BitVec` length or capacity. It
+	/// operates on the underlying `Vec`’s memory buffer directly, and ignores
+	/// the `BitVec`’s cursors.
 	///
-	/// This has the unobservable effect of setting the allocated, but dead,
-	/// bits beyond the end of the vector’s *length*, up to its *capacity*.
+	/// It is an implementation detail as to whether this affects the value of
+	/// allocated, but not currently used, elements in the buffer. Behavior of
+	/// this method on elements not visible through `self.as_slice()` is not
+	/// specified.
 	///
 	/// # Parameters
 	///
 	/// - `&mut self`
-	/// - `element`: The value to which each allocated element in the backing
-	///   store will be set.
+	/// - `element`: The value to which each live element in the backing store
+	///   will be set.
 	///
 	/// # Examples
 	///
@@ -528,17 +588,15 @@ where C: Cursor, T: BitStore {
 	/// use bitvec::prelude::*;
 	///
 	/// let mut bv = bitvec![Local, u8; 0; 10];
-	/// assert_eq!(bv.as_slice(), &[0, 0]);
+	/// //  note: the second element is not required to be zero, only to have
+	/// //  bits `0` and `1` according to `Local` be `0`.
+	/// assert_eq!(bv.as_slice()[0], 0);
 	/// bv.set_elements(0xA5);
 	/// assert_eq!(bv.as_slice(), &[0xA5, 0xA5]);
 	/// ```
+	#[inline]
 	pub fn set_elements(&mut self, element: T) {
-		self.do_unto_vec(|v| {
-			let (ptr, cap) = (v.as_mut_ptr(), v.capacity());
-			for elt in unsafe { slice::from_raw_parts_mut(ptr, cap) } {
-				*elt = element;
-			}
-		})
+		self.as_mut_slice().iter_mut().for_each(|elt| *elt = element);
 	}
 
 	/// Performs “reverse” addition (left to right instead of right to left).
@@ -634,7 +692,7 @@ where C: Cursor, T: BitStore {
 		}
 	}
 
-	/// Changes the cursor type on the vector handle, without changing its
+	/// Changes the order type on the vector handle, without changing its
 	/// contents.
 	///
 	/// # Parameters
@@ -643,13 +701,13 @@ where C: Cursor, T: BitStore {
 	///
 	/// # Returns
 	///
-	/// An equivalent vector handle with a new cursor type. The contents of the
+	/// An equivalent vector handle with a new order type. The contents of the
 	/// backing storage are unchanged.
 	///
 	/// To reorder the bits in memory, drain this vector into a new handle with
-	/// the desired cursor type.
-	pub fn change_cursor<D>(self) -> BitVec<D, T>
-	where D: Cursor {
+	/// the desired order type.
+	pub fn change_order<P>(self) -> BitVec<P, T>
+	where P: BitOrder {
 		let (bp, cap) = (self.pointer, self.capacity);
 		mem::forget(self);
 		unsafe { BitVec::from_raw_parts(bp, cap) }
@@ -664,11 +722,11 @@ where C: Cursor, T: BitStore {
 	/// # Returns
 	///
 	/// Itself, with its size frozen and ungrowable.
-	pub fn into_boxed_bitslice(self) -> BitBox<C, T> {
+	pub fn into_boxed_bitslice(self) -> BitBox<O, T> {
 		let pointer = self.pointer;
 		//  Convert the Vec allocation into a Box<[T]> allocation
 		mem::forget(self.into_boxed_slice());
-		unsafe { BitBox::from_raw(pointer) }
+		unsafe { BitBox::from_raw(pointer.as_mut_ptr()) }
 	}
 
 	/// Degrades a `BitVec` to a standard `Vec`.
@@ -689,6 +747,52 @@ where C: Cursor, T: BitStore {
 		out
 	}
 
+	/// Ensures that the live region of the underlying memory begins at the `0`
+	/// bit position.
+	///
+	/// # Notes
+	///
+	/// This method is currently implemented as a linear traversal that moves
+	/// each bit individually from its original index to its final position.
+	/// This is `O(n)` in the bit length of the vector.
+	///
+	/// It is possible to create an optimized rotation behavior that only moves
+	/// a few bits individually, then moves elements in a gallop. The speed
+	/// difference is proportional to the width of the element type.
+	///
+	/// When this behavior is implemented, `force_align` will be rewritten to
+	/// take advantage of it. For now, it remains slow.
+	///
+	/// # Examples
+	///
+	/// ```rust
+	/// use bitvec::prelude::*;
+	///
+	/// let src = &bits![Msb0, u8; 1, 0, 1, 1, 0, 1, 1, 0][1 .. 7];
+	/// assert_eq!(src.len(), 6);
+	/// let mut bv = src.to_owned();
+	/// assert_eq!(bv.len(), 6);
+	/// assert_eq!(bv.as_slice()[0], 0xB6);
+	/// bv.force_align();
+	/// assert_eq!(bv.as_slice()[0], 0x6E);
+	/// ```
+	pub fn force_align(&mut self) {
+		let (_, head, bits) = self.pointer.raw_parts();
+		let head = *head as usize;
+		if head == 0 {
+			return;
+		}
+		let tail = head + bits;
+		unsafe {
+			self.pointer.set_head(0.idx());
+			self.pointer.set_len(tail);
+			for (to, from) in (head .. tail).enumerate() {
+				self.copy_unchecked(from, to);
+			}
+			self.pointer.set_len(bits);
+		}
+	}
+
 	/// Permits a function to modify the `Vec<T>` underneath a `BitVec<_, T>`.
 	///
 	/// This produces a `Vec<T>` structure referring to the same data region as
@@ -707,7 +811,7 @@ where C: Cursor, T: BitStore {
 	///   closure) which receives a mutable borrow of a `Vec<T>`.
 	///
 	/// - `R`: The return value from the called function or closure.
-	fn do_unto_vec<F, R>(&mut self, func: F) -> R
+	fn with_vec<F, R>(&mut self, func: F) -> R
 	where F: FnOnce(&mut Vec<T>) -> R {
 		let slice = self.pointer.as_mut_slice();
 		let mut v = unsafe {
