@@ -3,7 +3,7 @@
 use super::*;
 
 use crate::{
-	cursor::Cursor,
+	order::BitOrder,
 	pointer::BitPtr,
 	store::BitStore,
 };
@@ -17,22 +17,20 @@ use core::{
 	cmp,
 	hint::unreachable_unchecked,
 	ops::RangeBounds,
-	ptr::{
-		self,
-		NonNull,
-	},
+	ptr::NonNull,
 };
 
-impl<C, T> BitVec<C, T>
-where C: Cursor, T: BitStore {
+impl<O, T> BitVec<O, T>
+where O: BitOrder, T: BitStore {
 	/// Constructs a new, empty `BitVec<C, T>`.
 	///
 	/// The vector will not allocate until elements are pushed onto it.
 	///
 	/// ```rust
 	/// # use bitvec::prelude::*;
-	/// let mut bv: BitVec<Local, Word> = BitVec::new();
+	/// let mut bv: BitVec<Local, usize> = BitVec::new();
 	/// ```
+	#[inline]
 	pub /* const */ fn new() -> Self {
 		Self::with_capacity(0)
 	}
@@ -58,7 +56,7 @@ where C: Cursor, T: BitStore {
 		//  Disarm the `Vec` destructor.
 		mem::forget(v);
 		Self {
-			_cursor: PhantomData,
+			_order: PhantomData,
 			pointer: BitPtr::uninhabited(ptr),
 			capacity: cap,
 		}
@@ -70,8 +68,9 @@ where C: Cursor, T: BitStore {
 	///
 	/// ```rust
 	/// # use bitvec::prelude::*;
-	/// let bv: BitVec<Local, Word> = BitVec::with_capacity(100);
+	/// let bv: BitVec<Local, usize> = BitVec::with_capacity(100);
 	/// assert!(bv.capacity() >= 100);
+	#[inline]
 	pub fn capacity(&self) -> usize {
 		self.capacity
 			.checked_mul(T::BITS as usize)
@@ -106,7 +105,7 @@ where C: Cursor, T: BitStore {
 		);
 		let (total_elts, _) = self.pointer.head().span(newlen);
 		if let Some(extra) = total_elts.checked_sub(self.pointer.elements()) {
-			self.do_unto_vec(|v| v.reserve(extra));
+			self.with_vec(|v| v.reserve(extra));
 		}
 	}
 
@@ -141,7 +140,7 @@ where C: Cursor, T: BitStore {
 		);
 		let (total_elts, _) = self.pointer.head().span(newlen);
 		let extra = total_elts - self.capacity;
-		self.do_unto_vec(|v| v.reserve_exact(extra));
+		self.with_vec(|v| v.reserve_exact(extra));
 	}
 
 	/// Shrinks the capacity of the vector as much as possible.
@@ -153,14 +152,15 @@ where C: Cursor, T: BitStore {
 	///
 	/// ```rust
 	/// # use bitvec::prelude::*;
-	/// let mut bv: BitVec<Local, Word> = BitVec::with_capacity(10);
+	/// let mut bv: BitVec<Local, usize> = BitVec::with_capacity(10);
 	/// bv.extend([true, false, true].iter().copied());
 	/// assert!(bv.capacity() >= 10);
 	/// bv.shrink_to_fit();
 	/// assert!(bv.capacity() >= 3);
 	/// ```
+	#[inline]
 	pub fn shrink_to_fit(&mut self) {
-		self.do_unto_vec(Vec::shrink_to_fit);
+		self.with_vec(Vec::shrink_to_fit);
 	}
 
 	/// Converts the bit-vector into [`Box<[T]>`].
@@ -183,12 +183,12 @@ where C: Cursor, T: BitStore {
 	///
 	/// ```rust
 	/// # use bitvec::prelude::*;
-	/// let mut bv = BitVec::<Local, Word>::with_capacity(100);
+	/// let mut bv = BitVec::<Local, usize>::with_capacity(100);
 	/// bv.extend([true, false, true].iter().copied());
 	///
 	/// assert!(bv.capacity() >= 100);
 	/// let slice = bv.into_boxed_slice();
-	/// let boxed_bitslice = BitBox::<Local, Word>::from_boxed_slice(slice);
+	/// let boxed_bitslice = BitBox::<Local, usize>::from_boxed_slice(slice);
 	/// let bv = BitVec::from_boxed_bitslice(boxed_bitslice);
 	/// assert!(bv.capacity() >= 3);
 	/// ```
@@ -196,6 +196,7 @@ where C: Cursor, T: BitStore {
 	/// [`BitBox<C, T>`]: ../boxed/struct.BitBox.html
 	/// [`Box<[T]>`]: https://doc.rust-lang.org/std/boxed/struct.Box.html
 	/// [`into_boxed_bitslice`]: #method.into_boxed_bitslice
+	#[inline]
 	pub fn into_boxed_slice(self) -> Box<[T]> {
 		self.into_vec().into_boxed_slice()
 	}
@@ -240,6 +241,7 @@ where C: Cursor, T: BitStore {
 	/// bv.truncate(0);
 	/// assert!(bv.is_empty());
 	/// ```
+	#[inline]
 	pub fn truncate(&mut self, len: usize) {
 		if len < self.len() {
 			unsafe { self.set_len(len) }
@@ -263,6 +265,7 @@ where C: Cursor, T: BitStore {
 	/// ```
 	///
 	/// [`BitSlice::as_slice`]: ../slice/struct.BitSlice.html#method.as_slice
+	#[inline]
 	pub fn as_slice(&self) -> &[T] {
 		self.pointer.as_slice()
 	}
@@ -285,6 +288,7 @@ where C: Cursor, T: BitStore {
 	/// ```
 	///
 	/// [`BitSlice::as_mut_slice`]: ../slice/struct.BitSlice.html#method.as_mut_slice
+	#[inline]
 	pub fn as_mut_slice(&mut self) -> &mut [T] {
 		self.pointer.as_mut_slice()
 	}
@@ -308,7 +312,7 @@ where C: Cursor, T: BitStore {
 	///
 	/// ```rust
 	/// # use bitvec::prelude::*;
-	/// let mut bv = BitVec::<Local, Word>::with_capacity(17);
+	/// let mut bv = BitVec::<Local, usize>::with_capacity(17);
 	/// assert!(bv.is_empty());
 	/// unsafe { bv.set_len(23) };
 	/// assert_eq!(bv.len(), 23);
@@ -476,12 +480,14 @@ where C: Cursor, T: BitStore {
 		//  If self is empty *or* tail is at the back edge of an element, push
 		//  an element onto the vector.
 		if self.is_empty() || *self.pointer.tail() == T::BITS {
-			self.do_unto_vec(|v| v.push(0.into()));
+			self.with_vec(|v| v.push(T::FALSE));
 		}
 		//  At this point, it is always safe to increment the tail, and then
 		//  write to the newly live bit.
-		unsafe { self.pointer.incr_tail() };
-		self.set(len, value);
+		unsafe {
+			self.pointer.set_len(len + 1);
+			self.set_unchecked(len, value);
+		}
 	}
 
 	/// Removes the last element from a vector and returns it, or `None` if it
@@ -502,12 +508,11 @@ where C: Cursor, T: BitStore {
 	/// assert!(bv.pop().is_none());
 	/// ```
 	pub fn pop(&mut self) -> Option<bool> {
-		if self.is_empty() {
-			return None;
-		}
-		let out = self[self.len() - 1];
-		unsafe { self.pointer.decr_tail() };
-		Some(out)
+		self.len().checked_sub(1).map(|new_len| unsafe {
+			let out = *self.get_unchecked(new_len);
+			self.set_len(new_len);
+			out
+		})
 	}
 
 	/// Moves all the elements of `other` into `self`, leaving `other` empty.
@@ -528,8 +533,9 @@ where C: Cursor, T: BitStore {
 	/// assert!(bv1[10]);
 	/// assert!(bv2.is_empty());
 	/// ```
+	#[inline]
 	pub fn append<D, U>(&mut self, other: &mut BitVec<D, U>)
-	where D: Cursor, U: BitStore {
+	where D: BitOrder, U: BitStore {
 		self.extend(other.iter().copied());
 		other.clear();
 	}
@@ -561,7 +567,7 @@ where C: Cursor, T: BitStore {
 	/// assert!(bv.not_any());
 	/// assert_eq!(bv.len(), 4);
 	/// ```
-	pub fn drain<R>(&mut self, range: R) -> Drain<C, T>
+	pub fn drain<R>(&mut self, range: R) -> Drain<O, T>
 	where R: RangeBounds<usize> {
 		use core::ops::Bound::*;
 		let len = self.len();
@@ -580,7 +586,7 @@ where C: Cursor, T: BitStore {
 		assert!(upto <= len, "The drain end must be within the vector bounds");
 
 		unsafe {
-			let ranging: &BitSlice<C, T> = self.as_bitslice()[from..upto]
+			let ranging: &BitSlice<O, T> = self.as_bitslice()[from..upto]
 				//  remove the lifetime and borrow awareness
 				.bitptr()
 				.into_bitslice();
@@ -642,14 +648,10 @@ where C: Cursor, T: BitStore {
 		let len = self.len();
 		assert!(at <= len, "Index out of bounds: {} is beyond {}", at, len);
 		match at {
-			0 => unsafe {
-				let out = Self::from_raw_parts(self.pointer, self.capacity);
-				ptr::write(self, Self::new());
-				out
-			},
+			0 => mem::replace(self, Self::new()),
 			n if n == len => Self::new(),
 			_ => {
-				let out = self.as_bitslice().iter().skip(at).copied().collect();
+				let out = self[at ..].to_owned();
 				self.truncate(at);
 				out
 			},
@@ -749,13 +751,13 @@ where C: Cursor, T: BitStore {
 	/// ```rust
 	/// # use bitvec::prelude::*;
 	/// let mut bv = bitvec![1];
-	/// bv.extend_from_slice(0xA5u8.bits::<LittleEndian>());
+	/// bv.extend_from_slice(0xA5u8.bits::<Lsb0>());
 	/// assert_eq!(bv, bitvec![1, 1, 0, 1, 0, 0, 1, 0, 1]);
 	/// ```
 	///
 	/// [`extend`]: #method.extend
 	pub fn extend_from_slice<D, U>(&mut self, other: &BitSlice<D, U>)
-	where D: Cursor, U: BitStore {
+	where D: BitOrder, U: BitStore {
 		let len = self.len();
 		let olen = other.len();
 		self.reserve(olen);
@@ -807,7 +809,7 @@ where C: Cursor, T: BitStore {
 		&mut self,
 		range: R,
 		replace_with: I,
-	) -> Splice<C, T, <I as IntoIterator>::IntoIter>
+	) -> Splice<O, T, <I as IntoIterator>::IntoIter>
 	where I: IntoIterator<Item=bool>, R: RangeBounds<usize> {
 		Splice {
 			drain: self.drain(range),
